﻿using MusicBrowser.ObjectModel;
using Semiodesk.Trinity;
using Semiodesk.Trinity.Collections;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace MusicBrowser
{
    public partial class MusicBrowserForm : Form
    {
        #region Members
        IModel _model;
        IStore _store;
        #endregion

        #region Constructor
        public MusicBrowserForm()
        {
            InitializeComponent();
           
            _store = StoreFactory.CreateStore("provider=sparqlendpoint;endpoint=http://linkedbrainz.org/sparql");
            _model = _store.GetModel(new Uri("http://linkedbrainz.org/sparql"));

            albumListBox.DisplayMember = "Title";
            trackListBox.MouseDoubleClick += trackListBox_MouseDoubleClick;
        }
        
        #endregion

        #region Methods
        async void MusicBrowserForm_StartSearch(object sender, EventArgs a)
        {
            Task<MusicArtist> artistTask = Task.Run(() => QueryForArtist(artistNameTextBox.Text));
            MusicArtist artist = await artistTask;

            if (artist != null)
                albumListBox.DataSource = CreatedReleases(artist);
            else
                MessageBox.Show(string.Format("No artist \"{0}\" found.", artistNameTextBox.Text));
        }

        MusicArtist QueryForArtist(string artist)
        {
            ResourceQuery query = new ResourceQuery(mo.MusicArtist);
            query.Where(foaf.name, artist);

            var res = _model.ExecuteQuery(query);
            var x = res.GetResources<MusicArtist>().ToList();
            if (x.Count > 0)
                return x.First();
            return null;
        }

        VirtualizingCollection<Release> CreatedReleases(MusicArtist artist)
        {
            ResourceQuery artistEntity = new ResourceQuery(artist);
            ResourceQuery madeEntity = new ResourceQuery(mo.Release);
            artistEntity.Where(foaf.made, madeEntity);
            return new VirtualizingResourceCollection<Release>(artist.Model, madeEntity);
        }

        async void albumListBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            trackListBox.DataSource = null;
            trackListBox.Enabled = false;

            Release item = albumListBox.SelectedItem as Release;
            if (item != null)
            {
                Task<List<Track>> tracksTask = new Task<List<Track>>(() =>
                {
                    if( item.Record.Count > 0)
                        return item.Record[0].Tracks.OrderBy((x) => x.TrackNumber).ToList();
                    return new List<Track>();
                });
                tracksTask.Start();
                
                trackListBox.DataSource = await tracksTask; ;
                trackListBox.DisplayMember = "Title";
                trackListBox.Enabled = true;
            }
        }

        void trackListBox_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            int index = this.trackListBox.IndexFromPoint(e.Location);
            if (index != System.Windows.Forms.ListBox.NoMatches)
            {
                Track item = trackListBox.Items[index] as Track;
                if (item != null)
                    System.Diagnostics.Process.Start(item.Uri.AbsoluteUri);
            }
        }
        #endregion
    }
}